package com.example.nekontakte.posts.api;

import com.example.nekontakte.core.api.PageDto;
import com.example.nekontakte.core.api.PageDtoMapper;
import com.example.nekontakte.core.configurations.Constants;

import org.apache.coyote.BadRequestException;
import org.modelmapper.ModelMapper;
import org.springframework.data.domain.PageRequest;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.example.nekontakte.posts.model.PostEntity;
import com.example.nekontakte.posts.service.PostService;
import com.example.nekontakte.users.service.UserService;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/post")
public class PostController {

    private final PostService postService;
    private final ModelMapper modelMapper;

    public PostController(PostService postService, ModelMapper modelMapper, UserService userService) {
        this.modelMapper = modelMapper;
        this.postService = postService;
    }

    private PostEntity toEntity(PostDTO dto) {
        return modelMapper.map(dto, PostEntity.class);
    }

    private PostDTO toDTO(PostEntity entity) {
        return modelMapper.map(entity, PostDTO.class);
    }

    @GetMapping
    public PageDto<PostDTO> getAll(@RequestParam(name = "pageNumber", defaultValue = "0") Integer pageNumber,
            @RequestParam(name = "pageSize", defaultValue = "5") Integer pageSize) {
        PageRequest pageRequest = PageRequest.of(pageNumber, pageSize);
        return PageDtoMapper.toDto(postService.getAll(pageRequest), this::toDTO);
    }

    @GetMapping("/user/{userId}")
    public PageDto<PostDTO> getAllByUserId(@PathVariable(name = "userId") Integer userId,
            @RequestParam(name = "pageNumber", defaultValue = "0") Integer pageNumber,
            @RequestParam(name = "pageSize", defaultValue = "5") Integer pageSize) {
        PageRequest pageRequest = PageRequest.of(pageNumber, pageSize);
        return PageDtoMapper.toDto(postService.getAllByUserId(userId, pageRequest), this::toDTO);
    }

    @GetMapping("/{id}")
    public PostDTO get(@PathVariable(name = "id") Integer id) {
        return toDTO(postService.get(id));
    }

    @PostMapping
    public PostDTO create(@RequestBody @Valid PostDTO PostDTO) throws BadRequestException {
        return toDTO(postService.create(toEntity(PostDTO)));
    }

    @PutMapping("/{id}")
    public PostDTO update(@PathVariable(name = "id") Integer id, @RequestBody PostDTO PostDTO) {
        return toDTO(postService.update(id, toEntity(PostDTO)));
    }

    @DeleteMapping("/{id}")
    public PostDTO delete(@PathVariable(name = "id") Integer id) {
        return toDTO(postService.delete(id));
    }

}
