package com.example.nekontakte.subscribes.service;

import java.util.List;

import java.util.stream.StreamSupport;

import org.apache.coyote.BadRequestException;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.nekontakte.core.errors.NotFoundException;
import com.example.nekontakte.subscribes.model.SubscribeEntity;
import com.example.nekontakte.subscribes.repository.SubscribeRepository;

@Service
public class SubscribeService {
    private final SubscribeRepository repository;

    public SubscribeService(SubscribeRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<SubscribeEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public Page<SubscribeEntity> getAll(PageRequest pageRequest) {
        return repository.findAll(pageRequest);
    }

    @Transactional(readOnly = true)
    public List<SubscribeEntity> getAllByUserId(Integer userId) {
        return repository.findByUserId(userId);
    }

    @Transactional(readOnly = true)
    public Page<SubscribeEntity> getAllByUserId(Integer userId, PageRequest pageRequest) {
        return repository.findByUserId(userId, pageRequest);
    }

    @Transactional(readOnly = true)
    public List<SubscribeEntity> getAllBySubscriberId(Integer subscriberId) {
        return repository.findBySubscriberId(subscriberId);
    }

    @Transactional(readOnly = true)
    public Page<SubscribeEntity> getAllBySubscriberId(Integer subscriberId, PageRequest pageRequest) {
        return repository.findBySubscriberId(subscriberId, pageRequest);
    }

    @Transactional(readOnly = true)
    public SubscribeEntity get(Integer id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(SubscribeEntity.class, id));
    }

    public SubscribeEntity create(SubscribeEntity entity) throws BadRequestException {
        if (entity.getSubscriber().getId() == entity.getUser().getId()) {
            throw new org.apache.coyote.BadRequestException("User do not subscribe themself");
        }
        return repository.save(entity);
    }

    public SubscribeEntity update(Integer id, SubscribeEntity entity) {
        final SubscribeEntity existEntity = get(id);
        existEntity.setId(entity.getId());
        existEntity.setSubscriber(entity.getSubscriber());
        existEntity.setUser(entity.getUser());
        return repository.save(existEntity);
    }

    public SubscribeEntity delete(Integer id) {
        final SubscribeEntity existsentity = get(id);
        repository.delete(existsentity);
        return existsentity;
    }
}
