package com.example.nekontakte.users.api;

import java.util.Objects;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.example.nekontakte.core.configurations.Constants;
import com.example.nekontakte.users.model.UserEntity;
import com.example.nekontakte.users.service.UserService;

import jakarta.validation.Valid;

@Controller
@RequestMapping(UserSignupController.URL)
public class UserSignupController {
    public static final String URL = "/signup";

    private static final String SIGNUP_VIEW = "signup";
    private static final String USER_ATTRIBUTE = "user";

    private final UserService userService;

    public UserSignupController(
            UserService userService) {
        this.userService = userService;
    }

    @GetMapping
    public String getSignup(Model model) {
        model.addAttribute(USER_ATTRIBUTE, new UserSignupDto());
        return SIGNUP_VIEW;
    }

    @PostMapping
    public String signup(
            @ModelAttribute(name = USER_ATTRIBUTE) @Valid UserSignupDto user,
            BindingResult bindingResult,
            Model model) {

        UserEntity userEntity = new UserEntity();
        userEntity.setUsername(user.getUsername());
        userEntity.setPassword(user.getPassword());

        if (userService.checkUserWithSameUsernameExist(userEntity)) {
            bindingResult.rejectValue("username", "signup:username", "Пользователь с таким логином уже есть");
        }

        if (user.getPassword() == null || user.getPassword() == "") {
            bindingResult.rejectValue("password", "signup:passwords", "Укажите пароль");
        }

        if (!Objects.equals(user.getPassword(), user.getPasswordConfirm())) {
            bindingResult.rejectValue("passwordConfirm", "signup:passwords", "Пароли не совпадают");
        }

        if (bindingResult.hasErrors()) {
            model.addAttribute(USER_ATTRIBUTE, user);
            return SIGNUP_VIEW;
        }

        userService.create(userEntity);
        return Constants.REDIRECT_VIEW + Constants.LOGIN_URL + "?signup";
    }
}
