package com.example.nekontakte.users.service;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import com.example.nekontakte.core.errors.NotFoundException;
import com.example.nekontakte.core.security.UserPrincipal;
import com.example.nekontakte.users.model.UserEntity;
import com.example.nekontakte.users.repository.UserRepository;

import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.StreamSupport;

@Service
public class UserService implements UserDetailsService {
    private final UserRepository repository;

    public boolean checkUserWithSameUsernameExist(String username) {
        return repository.findByUsernameIgnoreCase(username).isPresent();
    }

    public boolean checkUserWithSameUsernameExist(UserEntity user) {
        var existUser = repository.findByUsernameIgnoreCase(user.getUsername());
        if (existUser.isEmpty())
            return false;
        if (user.getId() == null)
            return true;
        return (int) existUser.get().getId() != (int) user.getId();
    }

    public UserService(UserRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public Page<UserEntity> getAll(Integer pageNumber, Integer pageSize) {
        PageRequest pageRequest = PageRequest.of(pageNumber, pageSize);
        return repository.findAll(pageRequest);
    }

    @Transactional(readOnly = true)
    public Page<UserEntity> getAll(PageRequest pageRequest) {
        return repository.findAll(pageRequest);
    }

    @Transactional(readOnly = true)
    public List<UserEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public UserEntity get(Integer id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(UserEntity.class, id));
    }

    @Transactional
    public UserEntity create(UserEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        if (checkUserWithSameUsernameExist(entity.getUsername())) {
            throw new IllegalArgumentException(
                    String.format("User with username %s is already exists", entity.getUsername()));
        }
        return repository.save(entity);
    }

    @Transactional
    public UserEntity update(UserEntity entity) {
        final UserEntity existsentity = get(entity.getId());
        if (checkUserWithSameUsernameExist(entity)) {
            throw new IllegalArgumentException(
                    String.format("User with username %s is already exists", entity.getUsername()));
        }
        repository.save(entity);
        return existsentity;
    }

    @Transactional
    public UserEntity delete(Integer id) {
        final UserEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }

    @Transactional(readOnly = true)
    public UserEntity getByUsername(String username) {
        return repository.findByUsernameIgnoreCase(username)
                .orElseThrow(() -> new IllegalArgumentException("Invalid login"));
    }

    @Override
    @Transactional(readOnly = true)
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        final UserEntity existsUser = getByUsername(username);
        return new UserPrincipal(existsUser);
    }
}